<?php
/**
 * @package     Joomla.Plugin
 * @subpackage  User.joomla
 *
 * @copyright   Copyright (C) 2005 - 2021 Open Source Matters, Inc. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */
defined('_JEXEC') or die();

/**
 * Joomla User plugin
 *
 * @since  1.0
 */
class PlgUserSso extends JPlugin
{
	/** @var provider */
	private $provider;

	/** @var providerUserId */
	private $providerUserId;

	/**
	 * Constructor.
	 *
	 * @param   object  &$subject  The object to observe
	 * @param   array   $config    An optional associative array of configuration settings.
	 *
	 * @since   3.7.0
	 */
	public function __construct(&$subject, $config)
	{
		// Don't even register this plugin if JFBCFactory isn't loaded and available (the jfbcsystem plugin likely isn't enabled)
        if (class_exists('JFBCFactory'))
        {
			parent::__construct($subject, $config);
        }
	}

	/* Function used as a trigger after User login
     *
     * @param   MIXED  $user     user ID
     * @param   MIXED  $options  Options available
     *
     * @return  boolean true or false
     *
     * @since  1.0.0
     */
    public function onUserLogin($user, $options)
    {
    	$this->provider       = JFBCFactory::provider('Azure');
		$this->providerUserId = $this->provider->getProviderUserId();

    	if ($this->provider && $this->providerUserId)
        {
        	if ($user['email'])
            {
				$userId  = JUserHelper::getUserId($user['username']);
				$userObj = JFactory::getUser($userId);

				// Fetch SSO profile
            	$profile = $this->provider->profile->fetchProfile($this->providerUserId);

	 			$userFields = json_decode($this->params->get("field_mapping"), true);

	 			if (!empty($userFields))
	 			{
	 				// Build joomla custom fields
	 				foreach ($userFields as $key => $value)
	 				{
	 					$formData['com_fields'][$key] = $profile->get($value);
	 				}
	 			}

				$formData['name'] = $profile->get('displayName');

				// Validate before updating email & username
				$ssoUserEmail = $profile->get('userPrincipalName');

				if (!ES::model('Users')->getUserId('email', $ssoUserEmail) && ($ssoUserEmail != $userObj->email))
				{
					$formData['email'] = $ssoUserEmail;
				}

				if (!JUserHelper::getUserId($ssoUserEmail) && ($ssoUserEmail != $userObj->username))
				{
					$formData['username'] = $ssoUserEmail;
				}

				$formData['id'] = $userObj->id;

				// Save joomla user details
                $userObj->bind($formData);
                $userObj->save();

                // Save easysocial user details
				// Get current ES user obj.
				$my = ES::user($userId);

				$esuserfields = $this->create_field_arr($my, $profile);

				if (! empty($esuserfields))
				{
					// Bind the custom fields for the user.
					$my->bindCustomFields($esuserfields);
					$my->bind($esuserfields);
					$my->save();
				}
            }
        }
    }

	/**
	 * create field array as per easysocial format for storing custom field data
	 *
	 * @param   Object  $user     ES user obj
	 * @param   Object  $profile  SSO profile obj
	 *
	 * @return  mixed
	 *
	 * @since 1.0
	 */
	public function create_field_arr($user, $profile)
	{
		$userfields = array();
		$fields     = json_decode($this->params->get("es_field_mapping"), true);

		if (! empty($fields))
		{
			foreach ($fields as $key => $value)
			{
				$localfield = ES::model('Fields')->getCustomFields(
					array(
						'group' => SOCIAL_TYPE_USER, 'uid' => $user->profile_id, 'data' => true, 'dataId' => $user->id,
							'dataType' => SOCIAL_TYPE_USER, "key" => $key
					)
				);

				switch ($key)
				{
					case "JOOMLA_FULLNAME":
						$fullname     = $profile->get($value);
						$fobj         = new stdClass;
						$fobj->first  = $fullname;
						$fobj->middle = '';
						$fobj->last   = '';
						$fobj->name   = $fullname;
						$userfields[SOCIAL_FIELDS_PREFIX . $localfield['0']->id] = $fobj;

						break;
					default:
						$userfields[SOCIAL_FIELDS_PREFIX . $localfield['0']->id] = $profile->get($value);
				}
			}

			return $userfields;
		}
	}

	/**
	 * This method should handle any logout logic and report back to the subject
	 *
	 * @param   array  $user     Holds the user data.
	 * @param   array  $options  Array holding options (client, ...).
	 *
	 * @return  bool  True on success
	 *
	 * @since   1.5
	 */
	// public function onUserLogout($user, $options = array())
	// {
	// 	// Create provider object
	// 	$provider = JFBCFactory::provider("climateid");

	// 	$componentParams = JComponentHelper::getParams('com_jfbconnect');
	// 	$ssositeurl = $componentParams->get('ssositeurl', '');

	// 	// Get the current user
	// 	$url = $ssositeurl . '/api/user/logout';

	// 	try
	// 	{
	// 		$jdata = $provider->client->query($url);
	// 		$data = json_decode($jdata->body, true);

	// 		$options = "{DATE}\t{TIME}\t{USER}\t{DESC}";
	// 		$my = JFactory::getUser();

	// 		JLog::addLogger(
	// 		array(
	// 			'text_file' => 'passport.log', 'text_entry_format' => $options
	// 		), JLog::INFO, 'passport');

	// 		$logEntry = new JLogEntry('logout user', JLog::INFO);
	// 		$logEntry->user = $my->name . '(' . $my->id . ')';
	// 		$logEntry->desc = json_encode($data);
	// 		JLog::add($logEntry);
	// 	}
	// 	catch (Exception $e)
	// 	{
	// 		jimport('joomla.error.log');
	// 		$options = "{DATE}\t{TIME}\t{USER}\t{DESC}";
	// 		$my = JFactory::getUser();

	// 		JLog::addLogger(
	// 		array(
	// 			'text_file' => 'passport.log', 'text_entry_format' => $options
	// 		), JLog::ERROR, 'passport');

	// 		$logEntry = new JLogEntry('Unable to logout user', JLog::ERROR);
	// 		$logEntry->user = $my->name . '(' . $my->id . ')';
	// 		$logEntry->desc = json_encode($e->getMessage());
	// 		JLog::add($logEntry);
	// 	}
	// }
}
